;;Created by Daniel Borrajo
;;Domain for delivering pods to human workers in a depot by robots

(define (domain depots-robots)
    (:requirements :strips :typing)
  (:types cell robot pod human - object)
  (:predicates
   (right ?c1 - cell ?c2 - cell)
   (left ?c1 - cell ?c2 - cell)
   (up ?c1 - cell ?c2 - cell)
   (down ?c1 - cell ?c2 - cell)
   (empty ?c - cell)
   (at-pod ?p - pod ?c - cell)
   (at-robot ?r - robot ?c - cell)
   (free ?r - robot)
   (carries ?r - robot ?p - pod)
   (serves ?h - human ?c - cell)
   (used ?h - human ?p - pod))

  (:action move-right
	   :parameters (?r - robot ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (right ?c1 ?c2)
			      (empty ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(not (empty ?c2))
			(not (at-robot ?r ?c1))))
  (:action move-left
	   :parameters (?r - robot ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (left ?c1 ?c2)
			      (empty ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(not (empty ?c2))
			(not (at-robot ?r ?c1))))
  (:action move-up
	   :parameters (?r - robot ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (up ?c1 ?c2)
			      (empty ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(not (empty ?c2))
			(not (at-robot ?r ?c1))))
  (:action move-down
	   :parameters (?r - robot ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (down ?c1 ?c2)
			      (empty ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(not (empty ?c2))
			(not (at-robot ?r ?c1))))

  (:action take-right
	   :parameters (?r - robot ?p - pod ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (free ?r)
			      (right ?c1 ?c2)
			      (at-pod ?p ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(carries ?r ?p)
			(not (free ?r))
			(not (at-pod ?p ?c2))
			(not (at-robot ?r ?c1))))
  (:action take-left
	   :parameters (?r - robot ?p - pod ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (free ?r)
			      (left ?c1 ?c2)
			      (at-pod ?p ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(carries ?r ?p)
			(not (free ?r))
			(not (at-pod ?p ?c2))
			(not (at-robot ?r ?c1))))
  (:action take-up
	   :parameters (?r - robot ?p - pod ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (free ?r)
			      (up ?c1 ?c2)
			      (at-pod ?p ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(carries ?r ?p)
			(not (free ?r))
			(not (at-pod ?p ?c2))
			(not (at-robot ?r ?c1))))
  (:action take-down
	   :parameters (?r - robot ?p - pod ?c1 - cell ?c2 - cell)
	   :precondition (and (at-robot ?r ?c1)
			      (free ?r)
			      (down ?c1 ?c2)
			      (at-pod ?p ?c2))
	   :effect (and (at-robot ?r ?c2)
			(empty ?c1)
			(carries ?r ?p)
			(not (free ?r))
			(not (at-pod ?p ?c2))
			(not (at-robot ?r ?c1))))

  (:action drop
	   :parameters (?r - robot ?p - pod ?c - cell)
	   :precondition (and (at-robot ?r ?c)
			      (carries ?r ?p))
	   :effect (and (not (carries ?r ?p))
			(at-pod ?p ?c)
			(free ?r)))

  (:action uses
	   :parameters (?h - human ?p - pod ?c - cell)
	   :precondition (and (serves ?h ?c)
			      (at-pod ?p ?c))
	   :effect (and (used ?h ?p)))
  )
