(define (domain elevators-fast)
  (:requirements :typing :action-costs)
  (:types slow-elevator fast-elevator passenger count
         )

(:predicates 
	(passenger-at ?person - passenger ?floor - count)
	(boarded ?person - passenger ?lift - slow-elevator)
	(boarded-fast ?person - passenger ?lift - fast-elevator)
	(lift-at ?lift - slow-elevator ?floor - count)
	(lift-at-fast ?lift - fast-elevator ?floor - count)
	(reachable-floor ?lift - slow-elevator ?floor - count)
	(reachable-floor-fast ?lift - fast-elevator ?floor - count)
	(above ?floor1 - count ?floor2 - count)
	(passengers ?lift - slow-elevator ?n - count)
	(passengers-fast ?lift - fast-elevator ?n - count)
	(can-hold ?lift - slow-elevator ?n - count)
	(can-hold-fast ?lift - fast-elevator ?n - count)
	(next ?n1 - count ?n2 - count)
)

(:action move-up-slow
  :parameters (?lift - slow-elevator ?f1 - count ?f2 - count )
  :precondition (and (lift-at ?lift ?f1) (above ?f1 ?f2 ) (reachable-floor ?lift ?f2) )
  :effect (and (lift-at ?lift ?f2) (not (lift-at ?lift ?f1))
	       ))

(:action move-down-slow
  :parameters (?lift - slow-elevator ?f1 - count ?f2 - count )
  :precondition (and (lift-at ?lift ?f1) (above ?f2 ?f1 ) (reachable-floor ?lift ?f2) )
  :effect (and (lift-at ?lift ?f2) (not (lift-at ?lift ?f1))
	       ))

(:action move-up-fast
  :parameters (?lift - fast-elevator ?f1 - count ?f2 - count )
  :precondition (and (lift-at-fast ?lift ?f1) (above ?f1 ?f2 ) (reachable-floor-fast ?lift ?f2) )
  :effect (and (lift-at-fast ?lift ?f2) (not (lift-at-fast ?lift ?f1))
	       ))

(:action move-down-fast
  :parameters (?lift - fast-elevator ?f1 - count ?f2 - count )
  :precondition (and (lift-at-fast ?lift ?f1) (above ?f2 ?f1 ) (reachable-floor-fast ?lift ?f2) )
  :effect (and (lift-at-fast ?lift ?f2) (not (lift-at-fast ?lift ?f1))
	       ))

(:action board-slow
  :parameters (?p - passenger ?lift - slow-elevator ?f - count ?n1 - count ?n2 - count)
  :precondition (and  (lift-at ?lift ?f) (passenger-at ?p ?f) (passengers ?lift ?n1) (next ?n1 ?n2) (can-hold ?lift ?n2) )
  :effect (and (not (passenger-at ?p ?f)) (boarded ?p ?lift) (not (passengers ?lift ?n1)) (passengers ?lift ?n2) ))

(:action leave-slow 
  :parameters (?p - passenger ?lift - slow-elevator ?f - count ?n1 - count ?n2 - count)
  :precondition (and  (lift-at ?lift ?f) (boarded ?p ?lift) (passengers ?lift ?n1) (next ?n2 ?n1) )
  :effect (and (passenger-at ?p ?f) (not (boarded ?p ?lift)) (not (passengers ?lift ?n1)) (passengers ?lift ?n2) ))

(:action board-fast
  :parameters (?p - passenger ?lift - fast-elevator ?f - count ?n1 - count ?n2 - count)
  :precondition (and  (lift-at-fast ?lift ?f) (passenger-at ?p ?f) (passengers-fast ?lift ?n1) (next ?n1 ?n2) (can-hold-fast ?lift ?n2) )
  :effect (and (not (passenger-at ?p ?f)) (boarded-fast ?p ?lift) (not (passengers-fast ?lift ?n1)) (passengers-fast ?lift ?n2) ))

(:action leave-fast 
  :parameters (?p - passenger ?lift - fast-elevator ?f - count ?n1 - count ?n2 - count)
  :precondition (and  (lift-at-fast ?lift ?f) (boarded-fast ?p ?lift) (passengers-fast ?lift ?n1) (next ?n2 ?n1) )
  :effect (and (passenger-at ?p ?f) (not (boarded-fast ?p ?lift)) (not (passengers-fast ?lift ?n1)) (passengers-fast ?lift ?n2) ))

)

